@@ -1,7 +1,7 @@
 /*
  * module_utils.h - Module utilities
  *           Functions to help writing output modules for Speech Dispatcher
- * Copyright (C) 2003, 2004 Brailcom, o.p.s.
+ * Copyright (C) 2003, 2004, 2007 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -18,7 +18,7 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  *
- * $Id: module_utils.h,v 1.13 2006/07/11 16:12:27 hanke Exp $
+ * $Id: module_utils.h,v 1.19 2007-07-14 05:32:45 hanke Exp $
  */
 
 #include <semaphore.h>
@@ -52,6 +52,7 @@ configfile_t *configfile;
 configoption_t *module_dc_options;
 int module_num_dc_options;
 
+
 #define CLEAN_OLD_SETTINGS_TABLE()\
  msg_settings_old.rate = -101;\
  msg_settings_old.pitch = -101;\
@@ -59,8 +60,9 @@ int module_num_dc_options;
  msg_settings_old.punctuation_mode = -1;\
  msg_settings_old.spelling_mode = -1;\
  msg_settings_old.cap_let_recogn = -1;\
- msg_settings_old.language = NULL;\
- msg_settings_old.voice = NO_VOICE;
+ msg_settings_old.language = NULL;	\
+ msg_settings_old.voice = NO_VOICE;\
+ msg_settings_old.synthesis_voice = NULL;
 
 #define INIT_SETTINGS_TABLES()\
  module_dc_options = NULL;\
@@ -70,8 +72,9 @@ int module_num_dc_options;
  msg_settings.punctuation_mode = PUNCT_NONE;\
  msg_settings.spelling_mode = SPELLING_OFF;\
  msg_settings.cap_let_recogn = RECOGN_NONE;\
- msg_settings.language = strdup("en");\
+ msg_settings.language = NULL;\
  msg_settings.voice = MALE1;\
+ msg_settings.synthesis_voice = NULL;\
  CLEAN_OLD_SETTINGS_TABLE()
 
 #define DBG(arg...) \
@@ -99,8 +102,10 @@ int module_num_dc_options;
 
 int     module_load         (void);
 int     module_init         (char **status_info);
+VoiceDescription** module_list_voices(void);
 int     module_speak        (char *data, size_t bytes, EMessageType msgtype);
 int     module_stop         (void);
+VoiceDescription**     module_get_voices   (void);
 size_t  module_pause        (void);
 char*     module_is_speaking  (void);
 void    module_close        (int status);
@@ -113,7 +118,7 @@ void    module_close        (int status);
     }
 
 #define UPDATE_STRING_PARAMETER(value, setter) \
-  if (msg_settings_old.value == NULL \
+  if (msg_settings_old.value == NULL || msg_settings.value == NULL \
          || strcmp (msg_settings_old.value, msg_settings.value)) \
     { \
       if (msg_settings_old.value != NULL) \
@@ -172,6 +177,7 @@ char* do_char(void);
 char* do_key(void);
 void  do_stop(void);
 void  do_pause(void);
+char* do_list_voices(void);
 char* do_set(void);
 int do_quit(void);
 
@@ -316,13 +322,13 @@ configoption_t * add_config_option(configoption_t *options, int *num_config_opti
     else name = NULL; \
     module_dc_options = module_add_config_option(module_dc_options, \
                                      &module_num_dc_options, #name, \
-                                     ARG_STR, name ## _cb, NULL, 0);
+                                     ARG_STR, name ## _cb, NULL, 0); 
 
 #define MOD_OPTION_1_INT_REG(name, default) \
     name = default; \
     module_dc_options = module_add_config_option(module_dc_options, \
                                      &module_num_dc_options, #name, \
-                                     ARG_INT, name ## _cb, NULL, 0);
+                                     ARG_INT, name ## _cb, NULL, 0); 
 
 /* TODO: Switch this to real float, not /100 integer,
    as soon as DotConf supports floats */
