@@ -57,8 +57,13 @@ class CallbackType(object):
     """The resume event is reported right after speaking of a message
     was resumed after previous pause."""
 
-    
 class SSIPError(Exception):
+    """Common base class for exceptions during SSIP communication."""
+    
+class SSIPCommunicationError(SSIPError):
+    """Exception raised when trying to operate on a closed connection."""
+    
+class SSIPResponseError(Exception):
     def __init__(self, code, msg, data):
         Exception.__init__(self, "%s: %s" % (code, msg))
         self._code = code
@@ -73,8 +78,8 @@ class SSIPError(Exception):
         """Return server response error message as string."""
         return self._msg
 
-    
-class SSIPCommandError(SSIPError):
+
+class SSIPCommandError(SSIPResponseError):
     """Exception raised on error response after sending command."""
 
     def command(self):
@@ -82,14 +87,14 @@ class SSIPCommandError(SSIPError):
         return self._data
 
     
-class SSIPDataError(SSIPError):
+class SSIPDataError(SSIPResponseError):
     """Exception raised on error response after sending data."""
 
     def data(self):
         """Return the data which resulted in this error."""
         return self._data
 
-
+    
 class _SSIP_Connection:
     """Implemantation of low level SSIP communication."""
     
@@ -122,14 +127,14 @@ class _SSIP_Connection:
                                  name="SSIP client communication thread")
         self._communication_thread.start()
     
-
     def close(self):
         """Close the server connection, destroy the communication thread."""
-        # Read-write shutdown here is necessary,
-        # otherwise the socket.recv() function
-        # in the other thread won't return at last
-        # on some platforms
-        self._socket.shutdown(socket.SHUT_RDWR)
+        # Read-write shutdown here is necessary, otherwise the socket.recv()
+        # function in the other thread won't return at last on some platforms.
+        try:
+            self._socket.shutdown(socket.SHUT_RDWR)
+        except socket.error:
+            pass
         self._socket.close()
         # Wait for the other thread to terminate
         self._communication_thread.join()
@@ -209,6 +214,10 @@ class _SSIP_Connection:
     def _recv_response(self):
         """Read server response from the communication thread
         and return the triplet (code, msg, data)."""
+        # TODO: This check is dumb but seems to work.  The main thread
+        # hangs without it, when the Speech Dispatcher connection is lost.
+        if not self._communication_thread.isAlive():
+            raise SSIPCommunicationError
         self._ssip_reply_semaphore.acquire()
         # The list is sorted, read the first item
         response = self._com_buffer[0]
@@ -221,22 +230,26 @@ class _SSIP_Connection:
         Arguments can be of any data type -- they are all stringified before
         being sent to the server.
 
-        Return a triplet (code, msg, data), where 'code' is a numeric SSIP
+        Returns a triplet (code, msg, data), where 'code' is a numeric SSIP
         response code as an integer, 'msg' is an SSIP rsponse message as string
         and 'data' is a tuple of strings (all lines of response data) when a
         response contains some data.
         
-        An 'SSIPCommandError' exception is raised in case of non 2xx return
-        code.  For more information about server responses and codes, see SSIP
-        documentation.
-       
+        'SSIPCommandError' is raised in case of non 2xx return code.  See SSIP
+        documentation for more information about server responses and codes.
+
+        'IOError' is raised when the socket was closed by the remote side.
+        
         """
         if __debug__:
             if command in ('SET', 'CANCEL', 'STOP',):
                 assert args[0] in (Scope.SELF, Scope.ALL) \
                        or isinstance(args[0], int)
         cmd = ' '.join((command,) + tuple(map(str, args)))
-        self._socket.send(cmd + self._NEWLINE)
+        try:
+            self._socket.send(cmd + self._NEWLINE)
+        except socket.error:
+            raise SSIPCommunicationError("Speech Dispatcher connection lost.")
         code, msg, data = self._recv_response()
         if code/100 != 2:
             raise SSIPCommandError(code, msg, cmd)
@@ -247,9 +260,10 @@ class _SSIP_Connection:
 
         Returned value is the same as for 'send_command()' method.
 
-        An 'SSIPCommandError' exception is raised in case of non 2xx return
-        code.  For more information about server responses and codes, see SSIP
-        documentation.
+        'SSIPDataError' is raised in case of non 2xx return code. See SSIP
+        documentation for more information about server responses and codes.
+        
+        'IOError' is raised when the socket was closed by the remote side.
         
         """
         # Escape the end-of-data marker even if present at the beginning
@@ -259,7 +273,10 @@ class _SSIP_Connection:
         elif data == self._END_OF_DATA_MARKER:
             data = self._END_OF_DATA_MARKER_ESCAPED
         data = data.replace(self._END_OF_DATA, self._END_OF_DATA_ESCAPED)
-        self._socket.send(data + self._END_OF_DATA)
+        try:
+            self._socket.send(data + self._END_OF_DATA)
+        except socket.error:
+            raise SSIPCommunicationError("Speech Dispatcher connection lost.")
         code, msg, response_data = self._recv_response()
         if code/100 != 2:
             raise SSIPDataError(code, msg, data)
@@ -269,7 +286,6 @@ class _SSIP_Connection:
         """Register a callback function for handling asynchronous events.
 
         Arguments:
-
           callback -- a callable object (function) which will be called to
             handle asynchronous events (arguments described below).  Passing
             `None' results in removing the callback function and ignoring
@@ -359,17 +375,12 @@ class SSIPClient(object):
         """Initialize the instance and connect to the server.
 
         Arguments:
-
           name -- client identification string
-
           component -- connection identification string.  When one client opens
             multiple connections, this can be used to identify each of them.
-            
           user -- user identification string (user name).  When multi-user
             acces is expected, this can be used to identify their connections.
-          
           host -- server hostname or IP address as a string.
-          
           port -- server port as number or None.  If None, the default value is
             determined by the SPEECH_PORT attribute of this class.
         
@@ -377,21 +388,21 @@ class SSIPClient(object):
         Dispatcher documentation.
           
         """
-        self._conn = _SSIP_Connection(host, port or self.SPEECH_PORT)
+        self._conn = conn = _SSIP_Connection(host, port or self.SPEECH_PORT)
         full_name = '%s:%s:%s' % (user, name, component)
-        self._conn.send_command('SET', Scope.SELF, 'CLIENT_NAME', full_name)
-        result = self._conn.send_command('HISTORY', 'GET', 'CLIENT_ID')
-        self._client_id = int(result[2][0])
+        conn.send_command('SET', Scope.SELF, 'CLIENT_NAME', full_name)
+        code, msg, data = conn.send_command('HISTORY', 'GET', 'CLIENT_ID')
+        self._client_id = int(data[0])
         self._lock = threading.Lock()
         self._callbacks = {}
-        self._conn.set_callback(self._callback_handler)
+        conn.set_callback(self._callback_handler)
         for event in (CallbackType.INDEX_MARK,
                       CallbackType.BEGIN,
                       CallbackType.END,
                       CallbackType.CANCEL,
                       CallbackType.PAUSE,
                       CallbackType.RESUME):
-            self._conn.send_command('SET', 'self', 'NOTIFICATION', event, 'on')
+            conn.send_command('SET', 'self', 'NOTIFICATION', event, 'on')
 
     
     def __del__(self):
@@ -421,7 +432,6 @@ class SSIPClient(object):
         """Set the priority category for the following messages.
 
         Arguments:
-
           priority -- one of the 'Priority' constants.
 
         """
@@ -434,15 +444,12 @@ class SSIPClient(object):
         """Say given message.
 
         Arguments:
-
           text -- message text to be spoken.  This may be either a UTF-8
             encoded byte string or a Python unicode string.
-
           callback -- a callback handler for asynchronous event notifications.
             A callable object (function) which accepts one positional argument
             `type' and one keyword argument `index_mark'.  See below for more
             details.
-
           event_types -- a tuple of event types for which the callback should
             be called.  Each item must be one of `CallbackType' constants.
             None (the default value) means to handle all event types.  This
@@ -482,7 +489,6 @@ class SSIPClient(object):
         """Say given character.
 
         Arguments:
-
           char -- a character to be spoken.  Either a Python unicode string or
             a UTF-8 encoded byte string.
 
@@ -498,7 +504,6 @@ class SSIPClient(object):
         """Say given key name.
 
         Arguments:
-
           key -- the key name (as defined in SSIP); string.
 
         This method is non-blocking;  it just sends the command, given
@@ -511,7 +516,6 @@ class SSIPClient(object):
         """Output given sound_icon.
 
         Arguments:
-
           sound_icon -- the name of the sound icon as defined by SSIP; string.
 
         This method is non-blocking; it just sends the command, given message
@@ -524,7 +528,6 @@ class SSIPClient(object):
         """Immediately stop speaking and discard messages in queues.
 
         Arguments:
-
           scope -- see the documentaion of this class.
             
         """
@@ -535,7 +538,6 @@ class SSIPClient(object):
         """Immediately stop speaking the currently spoken message.
 
         Arguments:
-
           scope -- see the documentaion of this class.
         
         """
@@ -549,11 +551,10 @@ class SSIPClient(object):
         sentence).
 
         Arguments:
-
           scope -- see the documentaion of this class.
         
         """
-        self._conn.send_command('STOP', scope)
+        self._conn.send_command('PAUSE', scope)
 
     def resume(self, scope=Scope.SELF):
         """Resume speaking of the currently paused messages.
@@ -563,35 +564,63 @@ class SSIPClient(object):
         sentence).
 
         Arguments:
-
           scope -- see the documentaion of this class.
         
         """
-        self._conn.send_command('STOP', scope)
+        self._conn.send_command('RESUME', scope)
+
+    def list_output_modules(self):
+        """Return names of all active output modules as a tuple of strings."""
+        code, msg, data = self._conn.send_command('LIST', 'OUTPUT_MODULES')
+        return data
+
+    def list_synthesis_voices(self):
+        """Return names of all available voices for the current output module.
+
+        Returns a tuple of tripplets (name, language, dialect).
+
+        'name' is a string, 'language' is an ISO 639-1 Alpha-2 language code
+        and 'dialect' is a string.  Language and dialect may be None.
+
+        """
+        try:
+            code, msg, data = self._conn.send_command('LIST', 'SYNTHESIS_VOICES')
+        except SSIPCommandError:
+            return ()
+        def split(item):
+            name, lang, dialect = tuple(item.rsplit(' ', 3))
+            return (name, lang or None, dialect or None)
+        return tuple([split(item) for item in data])
 
     def set_language(self, language, scope=Scope.SELF):
         """Switch to a particular language for further speech commands.
 
         Arguments:
-
           language -- two letter language code according to RFC 1776 as string.
-
           scope -- see the documentaion of this class.
             
         """
         assert isinstance(language, str) and len(language) == 2
         self._conn.send_command('SET', scope, 'LANGUAGE', language)
 
+    def set_output_module(self, name, scope=Scope.SELF):
+        """Switch to a particular output module.
+
+        Arguments:
+          name -- module (string) as returned by 'list_output_modules()'.
+          scope -- see the documentaion of this class.
+        
+        """
+        self._conn.send_command('SET', scope, 'OUTPUT_MODULE', name)
+
     def set_pitch(self, value, scope=Scope.SELF):
         """Set the pitch for further speech commands.
 
         Arguments:
-
           value -- integer value within the range from -100 to 100, with 0
             corresponding to the default pitch of the current speech synthesis
             output module, lower values meaning lower pitch and higher values
             meaning higher pitch.
-
           scope -- see the documentaion of this class.
           
         """
@@ -602,12 +631,10 @@ class SSIPClient(object):
         """Set the speech rate (speed) for further speech commands.
 
         Arguments:
-
           value -- integer value within the range from -100 to 100, with 0
             corresponding to the default speech rate of the current speech
             synthesis output module, lower values meaning slower speech and
             higher values meaning faster speech.
-
           scope -- see the documentaion of this class.
             
         """
@@ -618,11 +645,9 @@ class SSIPClient(object):
         """Set the speech volume for further speech commands.
 
         Arguments:
-
           value -- integer value within the range from -100 to 100, with 100
             corresponding to the default speech volume of the current speech
             synthesis output module, lower values meaning softer speech.
-
           scope -- see the documentaion of this class.
             
         """
@@ -633,9 +658,7 @@ class SSIPClient(object):
         """Set the punctuation pronounciation level.
 
         Arguments:
-
           value -- one of the 'PunctuationMode' constants.
-
           scope -- see the documentaion of this class.
             
         """
@@ -647,11 +670,9 @@ class SSIPClient(object):
         """Toogle the spelling mode or on off.
 
         Arguments:
-
           value -- if 'True', all incomming messages will be spelled
             instead of being read as normal words. 'False' switches
             this behavior off.
-
           scope -- see the documentaion of this class.
             
         """
@@ -665,12 +686,10 @@ class SSIPClient(object):
         """Set capital letter recognition mode.
 
         Arguments:
-
           value -- one of 'none', 'spell', 'icon'. None means no signalization
             of capital letters, 'spell' means capital letters will be spelled
             with a syntetic voice and 'icon' means that the capital-letter icon
             will be prepended before each capital letter.
-
           scope -- see the documentaion of this class.
             
         """
@@ -678,18 +697,16 @@ class SSIPClient(object):
         self._conn.send_command('SET', scope, 'CAP_LET_RECOGN', value)
 
     def set_voice(self, value, scope=Scope.SELF):
-        """Set voice to given value.
-
-        Note that this doesn't guarantee that such a voice is actually
-        available by the used synthesizer.  If not, the suggested voice will be
-        mapped to one of the available voices.
+        """Set voice by a symbolic name.
 
         Arguments:
-
           value -- one of the SSIP symbolic voice names: 'MALE1' .. 'MALE3',
             'FEMALE1' ... 'FEMALE3', 'CHILD_MALE', 'CHILD_FEMALE'
-
           scope -- see the documentaion of this class.
+
+        Symbolic voice names are mapped to real synthesizer voices in the
+        configuration of the output module.  Use the method
+        'set_synthesis_voice()' if you want to work with real voices.
             
         """
         assert isinstance(value, str) and \
@@ -698,14 +715,22 @@ class SSIPClient(object):
                                  "child_female")
         self._conn.send_command('SET', scope, 'VOICE', value)
 
+    def set_synthesis_voice(self, value, scope=Scope.SELF):
+        """Set voice by its real name.
+
+        Arguments:
+          value -- voice name as returned by 'list_synthesis_voices()'
+          scope -- see the documentaion of this class.
+            
+        """
+        self._conn.send_command('SET', scope, 'SYNTHESIS_VOICE', value)
+        
     def set_pause_context(self, value, scope=Scope.SELF):
         """Set the amount of context when resuming a paused message.
 
         Arguments:
-
           value -- a positive or negative value meaning how many chunks of data
             after or before the pause should be read when resume() is executed.
-
           scope -- see the documentaion of this class.
             
         """
